//
// DepthColoring effect
// Copyright (c) 2002 Alexey Barkovoy
//

// Depth coloring parameters
texture colTexture;
matrix  colMatrix;

matrix mTot; // Total matrix = World*View*Proj Matrix
matrix mWorldView;
matrix mWVColor; // World*View * TextureTransform(aka Color)

technique FixedFunction_with_texture
{
    pass P0
    {
        VertexShader = xyz;

        Lighting       = False;
        SpecularEnable = False;

        // Stage 0
        ColorOp[0]   = SelectArg1;
        ColorArg1[0] = Texture;
        ColorArg2[0] = Current;

        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Texture;
        AlphaArg2[0] = Current;

        MinFilter[0] = Linear;
        MagFilter[0] = Linear;
        MipFilter[0] = None;

        Texture[0]   = <colTexture>;
        TextureTransform[0] = <colMatrix>;
        TextureTransformFlags[0] = Count2;
        TexCoordIndex[0] = CameraSpacePosition;

        // Stage 1
        ColorOp[1]   = Disable;
        AlphaOp[1]   = Disable;
    }
}

technique VShader_Texture
{
    pass P0
    {
        VertexShader = xyz;

        Lighting       = False;
        SpecularEnable = False;

        // Stage 0
        ColorOp[0]   = SelectArg1;
        ColorArg1[0] = Texture;
        ColorArg2[0] = Current;

        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Texture;
        AlphaArg2[0] = Current;

        MinFilter[0] = Linear;
        MagFilter[0] = Linear;
        MipFilter[0] = None;

        // Stage 1
        ColorOp[1]   = Disable;
        AlphaOp[1]   = Disable;

        Texture[0]   = <colTexture>;

        VertexShaderConstant[0] = <mTot>; // Total matrix
        VertexShaderConstant[4] = <mWVColor>;

        // Definition of the vertex shader, declarations then assembly.
        VertexShader =
        decl
        {
            stream 0;
            float v0[3];        // Position
        }
        asm
        {
            vs.1.1             // version number

            m4x4 oPos, v0, c0  // transform point to projection space

            m4x4 r1, v0, c4
            mov oT0.x, r1.x
//          dp4 oT0.x, v0, c4 // - equal to 2 lines above (when optimized in driver...)
        };
    }
}

technique VShader_ColorCalc
{
    pass P0
    {
        Lighting       = False;
        SpecularEnable = False;

        // Stage 0
        ColorOp[0]   = SelectArg1;
        ColorArg1[0] = Diffuse;
        ColorArg2[0] = Current;

        AlphaOp[0]   = SelectArg1;
        AlphaArg1[0] = Diffuse;
        AlphaArg2[0] = Current;

        MinFilter[0] = Linear;
        MagFilter[0] = Linear;
        MipFilter[0] = None;

        // Stage 1
        ColorOp[1]   = Disable;
        AlphaOp[1]   = Disable;

        VertexShaderConstant[3] = <mTot>; // Total matrix
        VertexShaderConstant[7] = <mWVColor>;

        // Definition of the vertex shader, declarations then assembly.
        VertexShader =
        decl
        {
            stream 0;
            float v0[3];        // Position
        }
        asm
        {
// Registers:
//
//     Name         Reg   Size
//     ------------ ----- ----
//     mTotal       c3       4
//     mWVColor     c7       4/1

    vs.1.1             // version number

    def c0, 1, 5, -1, 6
    def c1, 1, 2, 3, 4
    def c2, 0, 0, 0, 0

    // Transform position
    m4x4 oPos, v0, c3

    // hue = Mul(vPos, mWVColor).x;
    dp4 r0.w, v0, c7
    // hue = 1 - hue;
    add r2.x, -r0.w, c0.x
    // frc(hue); // Scale this to HSL range
    frc r3.y, r2.x
    // part = frc(hue)*6; // Scale this to HSL range
    // rest = hue*6;
    mov r2.y, r3.y
    mul r9.xy, r2, c0.w

    // Compare <frc(hue)*6> to <1, 2, 3, 4>
    slt r4, r9.y, c1.wzyx

    mov r9.w, c0.x

    // rest = frc(hue*6);
    frc r3.y, r9.x
    mov r11.z, r3.y

    // 1-rest
    add r9.z, -r11.z, c0.x

    // Diffuse = (1, 0, 1-rest, 0)
    mov r11.y, -r9.z
    mov r11.xw, c0.zyzx
    add r11.xy, r11.zwzw, r11
    // Compare <frc(hue)*6> to <5>
    slt r9.y, r9.y, c0.y
    mad r9.xy, r9.y, r11, r9.wzzw       // r9

    mov r6.w, r9.z
    mov r6.xy, -r9.yxzw
    add r6.z, r6.x, c0.x
    mul r1.xyz, r4.x, r6.yzww
    add r8.xy, r9, r1                   // r8 = (BOOL*Col) + r9

    mov r8.z, r1.z
    mov r10.x, -r8.x
    add r10.yz, r11.xwzw, -r8.xzyw
    mad r0.xyz, r4.y, r10, r8.xzyw      // r0 = (BOOL*Col) + r8

    add r2.xy, r9.zwzw, -r0
    mov r2.z, -r0.z
    mad r6.xyz, r4.z, r2, r0            // r6 = (BOOL*Col) + r0

    add r11.xy, r11.wzzw, -r6
    mov r11.z, -r6.z
    mad oD0.xyz, r4.w, r11, r6          // Diffuse = (BOOL*Col) + r6

    mov oD0.w, c2.x

        };
    }
}
