(*----------------------------------------------------------------------------*
 *  Direct3D DepthColoring sample                                             *
 *  demostrate how to convert depth information to color during vertex        *
 *  processing. DirectX 8.1 headers and lib's are required.                   *
 *  Above mentioned effect is generated using 3 different methods:            *
 *  1) Good old texture matrix transformation using DX7 style fixed function  *
 *     T&L and accessing small pre-rendered texture while pixel shading.      *
 *  2) Modern style DX8 VertexShader version of first method - 40% faster on  *
 *     GF3 on complex meshes. Also accesses prerendered textule later.        *
 *  3) Calculating color (hue channel of HSL color model) completely in       *
 *     vertex shader. Pros: no texture needed - ouputs standart diffuse color;*
 *     Cons: more complex calculations required - less vertex processing speed*
 *  So, recommendations: on hardware capable of VS use (2) else (1), if you   *
 *  need many textures when (3) may be win for your setup.                    *
 *                                                                            *
 *  Note: This sample requires DirectX 8.1 compatible headers to compile !!!  *
 *                                                                            *
 *  Copyright (c) 2002 by Alexey Barkovoy                                     *
 *  E-Mail: clootie@reactor.ru                                                *
 *                                                                            *
 *  Modified: 14-Sep-2002                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru                                                *
 *----------------------------------------------------------------------------*)

program DepthColoring;

{%File 'Depth.fx'}
//{$R 'winmain.res' 'winmain.rc'}

uses
  Windows,
  DepthColoringUnit in 'DepthColoringUnit.pas',
  Resource in 'Resource.pas';

{$R winmain.res}

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything, and goes into a
//       message-processing loop. Idle time is used to render the scene.
//-----------------------------------------------------------------------------
var
  d3dApp: CMyD3DApplication;
begin
  d3dApp:= CMyD3DApplication.Create;
  // "Create_" isn't a constructor it's a regular function
  if FAILED(d3dApp.Create_(HInstance)) then Exit;
  d3dApp.Run;
end.

